#include <stdio.h>
#include <stdlib.h>
#include "mpi.h"
#include "fpmpi.h"

int main(int argc, char *argv[])
{
    int    wsize, wrank;
    int    b[2];
    int    src, dest, i, bufsize, ntrial=100;
    double *rbuf, *sbuf;
    MPI_Request r[2];

    MPI_Init(&argc, &argv);

    MPI_Comm_size(MPI_COMM_WORLD, &wsize);
    MPI_Comm_rank(MPI_COMM_WORLD, &wrank);

    /* A simple code to mock up some setup code */
    MPI_Barrier(MPI_COMM_WORLD);
#if 0
    if (wrank == 0) {
	b[0] = wsize;
	b[1] = -1;
	MPI_Send(b, 2, MPI_INT, wsize-1, 0, MPI_COMM_WORLD);
    }
    if (wrank == wsize-1) {
	MPI_Recv(b, 2, MPI_INT, 0, 0, MPI_COMM_WORLD, MPI_STATUS_IGNORE);
    }
#endif
    MPI_Pcontrol(FPMPI_PROF_END_PHASE, "setup");

    /* Simple communication in a ring */
    dest = (wrank + 1) % wsize;
    src  = (wsize + wrank - 1) % wsize;
    bufsize = 1000;
    rbuf = (double *)malloc(bufsize*sizeof(double));
    sbuf = (double *)malloc(bufsize*sizeof(double));
    for (i=0; i<ntrial; i++) {
	MPI_Irecv(rbuf, bufsize, MPI_DOUBLE, src, i, MPI_COMM_WORLD, &r[0]);
	MPI_Isend(sbuf, bufsize, MPI_DOUBLE, dest, i, MPI_COMM_WORLD, &r[1]);
	MPI_Waitall(2, r, MPI_STATUSES_IGNORE);
    }
    MPI_Pcontrol(FPMPI_PROF_END_PHASE, "compute");
    for (i=0; i<ntrial; i++) {
	int nleft;
	MPI_Irecv(rbuf, bufsize-1, MPI_DOUBLE, src, i, MPI_COMM_WORLD, &r[0]);
	MPI_Isend(sbuf, bufsize-1, MPI_DOUBLE, dest, i, MPI_COMM_WORLD, &r[1]);
	nleft = 2;
	while (nleft > 0) {
	    int idx, flag;
	    MPI_Testany(2, r, &idx, &flag, MPI_STATUS_IGNORE);
	    if (flag) nleft--;
	}
    }
    MPI_Pcontrol(FPMPI_PROF_END_PHASE, "poll");

    free(rbuf);
    free(sbuf);
    MPI_Finalize();
    return 0;
}
